<?php
namespace Application\Model;

use Zend\Db\Adapter\Adapter;
use Zend\Db\Sql\Predicate\Expression;
use Zend\Db\Sql\Sql;
use Zend\Db\Sql\Where;


/**
 * Class DomainModel
 * @package Application\Model
 */
class DomainModel
{
    const TABLE = 'ws_domain';

    /**
     * @var array
     */
    protected $_whitelist = array(
        'id',
        'fullname',
        'url',
        'id_provider',
        'id_hosting',
        'host_ftp',
        'user_ftp',
        'pwd_ftp',
        'port_ftp',
        'type_ftp',
        'auth_code',
        'expired',
        'info',
        'created',
        'id_customers',
        'id_server',
        'price',
    );

    /**
     * @var \Zend\Db\Adapter\Adapter
     */
    public $adapter;

    /**
     * @var \Zend\Db\Sql\Sql
     */
    public $sql;

    /**
     * @param Adapter $adapter
     */
    public function __construct(Adapter $adapter)
    {
        $this->adapter = $adapter;
        $this->sql     = new Sql($adapter);
    }

    /**
     * @param $data
     * @return mixed|null
     */
    public function insert($data)
    {
        $values = array_intersect_key($data, array_flip($this->_whitelist));

        $insert = $this->sql->insert(self::TABLE);
        $insert->values($values);

        $statement = $this->sql->prepareStatementForSqlObject($insert);
        return $statement->execute()->getGeneratedValue();
    }

    /**
     * @param $id
     * @param array $data
     * @return mixed|null
     */
    public function update($id, array $data)
    {
        $values = array_intersect_key($data, array_flip($this->_whitelist));

        $where = new \Zend\Db\Sql\Where();
        $where->equalTo('id', (int)$id);

        $update = $this->sql->update(self::TABLE);
        $update->set($values)->where($where);

        $statement = $this->sql->prepareStatementForSqlObject($update);
        return $statement->execute()->getGeneratedValue();
    }

    /**
     * @param $id
     * @return \Zend\Db\Adapter\Driver\ResultInterface
     */
    public function delete($id)
    {
        $delete = $this->sql->delete(self::TABLE);
        $delete->where(array(
            'id' => $id
        ));

        $deleteStatement = $this->sql->prepareStatementForSqlObject($delete);
        return $deleteStatement->execute();
    }

    /**
     * @param array $filterBy
     * @return \Zend\Db\Adapter\Driver\ResultInterface
     */
    public function getList(array $filterBy = array())
    {
        $select = $this->sql->select();
        $select->from(array('d' => self::TABLE));
        $select->join(array('p' => 'ws_provider'), 'p.id = d.id_provider', array('name_provider' => 'fullname'), 'left');
        $select->join(array('h' => 'ws_hosting'), 'h.id = d.id_hosting', array('name_hosting' => 'fullname'), 'left');
        $select->join(array('s' => 'ws_server'), 's.id = d.id_server', array('name_server' => 'fullname'), 'left');
        $select->join(array('c' => 'ws_customers'), 'c.id = d.id_customers', array('name_customers' => 'fullname'), 'left');
        $select->where($filterBy);
        $select->order('d.id ASC');

        // echo $this->sql->getSqlStringForSqlObject($select);
        $statement = $this->sql->prepareStatementForSqlObject($select);
        $dataset   = $statement->execute();
        return $dataset;
    }

    /**
     * @param array $filterBy
     * @return mixed
     */
    public function find(array $filterBy = array())
    {
        $select = $this->sql->select();
        $select->from(array('d' => self::TABLE));
        $select->join(array('p' => 'ws_provider'), 'p.id = d.id_provider', array('name_provider' => 'fullname'), 'left');
        $select->join(array('h' => 'ws_hosting'), 'h.id = d.id_hosting', array('name_hosting' => 'fullname'), 'left');
        $select->join(array('c' => 'ws_customers'), 'c.id = d.id_customers', array('name_customers' => 'fullname'), 'left');
        $select->join(array('s' => 'ws_server'), 's.id = d.id_server', array('name_server' => 'fullname'), 'left');
        $select->where($filterBy);

        // echo $this->sql->getSqlStringForSqlObject($select);
        $statement = $this->sql->prepareStatementForSqlObject($select);
        return $statement->execute()->current();
    }

    /**
     * @return mixed
     */
    public function count()
    {
        $select = $this->sql->select();
        $select->from(self::TABLE);
        $select->columns(array('min' => new \Zend\Db\Sql\Expression('COUNT(*)')));
        $statement = $this->sql->prepareStatementForSqlObject($select);
        return $statement->execute()->current();
    }

    /**
     * @return mixed
     */
    public function getLastExpiring()
    {
        $today    = new \DateTime('NOW');
        $interval = new \DateTime('7 days');

        $select = $this->sql->select();
        $select->from(array('d' => self::TABLE));
        $select->join(array('p' => 'ws_provider'), 'p.id = d.id_provider', array('name_provider' => 'fullname'), 'left');
        $select->join(array('c' => 'ws_customers'), 'c.id = d.id_customers', array('name_customers' => 'fullname'), 'left');

        $where = new Where();
        $where->between('d.expired', $today->format('Y-m-d'), $interval->format('Y-m-d'));
        $select->where($where);
        $select->order('d.id DESC');

        $statement = $this->sql->prepareStatementForSqlObject($select);
        return $statement->execute();
    }

    /**
     * @return \Zend\Db\Adapter\Driver\ResultInterface
     */
    public function getExpireDomains()
    {
        $today = new \DateTime('NOW');

        $select = $this->sql->select();
        $select->from(array('d' => self::TABLE));
        $select->join(array('p' => 'ws_provider'), 'p.id = d.id_provider', array('name_provider' => 'fullname'), 'left');
        $select->join(array('c' => 'ws_customers'), 'c.id = d.id_customers', array('name_customers' => 'fullname'), 'left');

        $where = new Where();
        $where->lessThanOrEqualTo('d.expired', $today->format('Y-m-d'));
        $select->where($where);
        $select->order('d.id DESC');

        $statement = $this->sql->prepareStatementForSqlObject($select);
        return $statement->execute();
    }

    /**
     * @param $type
     * @return int
     */
    public function getDomainChart($type)
    {
        $select = $this->sql->select();
        $select->from(array('d' => self::TABLE));

        switch ($type) {
            case 'active':
                $select->where->addPredicate(new Expression("expired > DATE('now') "));
                break;

            case 'expired':
                $select->where->addPredicate(new Expression("expired < DATE('now') "));
                break;
        }

        $statement = $this->sql->prepareStatementForSqlObject($select);
        return $statement->execute()->count();
    }
}